module PaymentOrchestration
  class Payment < ApplicationRecord
    serialize :payment_details, type: Hash, coder: YAML
    serialize :delivery_details, type: Hash, coder: YAML

    # Validations
    validates :payment_method, presence: true
    validates :status, inclusion: { in: %w[pending successful failed] }

    # Enum for payment_method
    enum :payment_method, {
      debit_or_credit_card: "debit_or_credit_card",
      paypal: "paypal",
      direct_debit: "direct_debit",
      apple_pay: "apple_pay",
      google_pay: "google_pay"
    } # , suffix: :type

    # Payment gateway configurations
    def self.payment_gateways
      [
        {
          method: :debit_or_credit_card,
          name: "Debit/Credit Card",
          gateway_class: "PaymentOrchestration::PaymentGateway::Stripe"
        },
        {
          method: :paypal,
          name: "PayPal",
          gateway_class: "PaymentOrchestration::PaymentGateway::PayPal"
        }
      ]
    end

    # Payment processing method
    def process_payment!(options = {})
      payment_gateway = determine_gateway
      transaction_result = payment_gateway.process(options)

      update!(
        status: transaction_result.success? ? "successful" : "failed",
        transaction_reference: transaction_result.reference,
        success_code: transaction_result.success_code || "UNKNOWN"
      )
    end

    private

    def determine_gateway
      gateway_class.constantize.new(
        payment_method: payment_method,
        transaction: transaction
      )
    end
  end
end
